#include "sequence_compare/hamming_distance.h"

#include <gmock/gmock.h>
#include <gtest/gtest.h>

namespace hamming_distance {
TEST(HammingDistance, CompareSequencesWorks) {
  // '-' is not counted as difference.
  {
    std::string seq_1 = "a-cg-ta";
    std::string seq_2 = "a-cgt-a";
    int sites_compared;
    EXPECT_TRUE(CompareSequences(seq_1, seq_2, &sites_compared).empty());
    EXPECT_EQ(sites_compared, 4);
  }

  // Count difference correclty.
  {
    std::string seq_1 = "accg-ta";
    std::string seq_2 = "atcgtga";
    int sites_compared;
    EXPECT_THAT(CompareSequences(seq_1, seq_2, &sites_compared),
                ::testing::ElementsAre(1, 5));
    EXPECT_EQ(sites_compared, 6);
  }
}

TEST(HammingDistance, WindowHammingDistanceWorks) {
  // Output correct number of elements.
  {
    std::string seq_1(300, 'a');
    std::string seq_2(300, 'a');
    EXPECT_EQ(WindowHammingDistance(seq_1, seq_2, 30, 5, false),
              std::vector<int>(55, 0));
    std::vector<int> expected(55, 0);
    expected.push_back(300);
    EXPECT_EQ(WindowHammingDistance(seq_1, seq_2, 30, 5, true), expected);
  }


  {
    std::string seq_1(7, 'a');
    std::string seq_2 = "attaata";
    std::string seq_3 = "atta-ta";

    // Dense output of difference.
    EXPECT_THAT(WindowHammingDistance(seq_1, seq_2, 1, 1, false),
                ::testing::ElementsAre(0, 1, 1, 0, 0, 1, 0));
    EXPECT_THAT(WindowHammingDistance(seq_1, seq_2, 1, 1, true),
                ::testing::ElementsAre(0, 1, 1, 0, 0, 1, 0, 7));

    // Count hamming distance correclty.
    EXPECT_THAT(WindowHammingDistance(seq_1, seq_2, 3, 2, false),
                ::testing::ElementsAre(2, 1, 1));
    EXPECT_THAT(WindowHammingDistance(seq_1, seq_2, 3, 2, true),
                ::testing::ElementsAre(2, 1, 1, 7));

    // Handles invalid site.
    EXPECT_THAT(WindowHammingDistance(seq_1, seq_3, 3, 2, false),
                ::testing::ElementsAre(2, 1, 1));
    EXPECT_THAT(WindowHammingDistance(seq_1, seq_3, 3, 2, true),
                ::testing::ElementsAre(2, 1, 1, 6));

  }
}
}  // namespace hamming_distance
